/*
 * ExpansionContractionPanel.java
 * Created on 2011/10/13
 *
 * Copyright (C) 2011-2013 Nippon Telegraph and Telephone Corporation
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

package jp.co.ntt.oss.heapstats.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.Border;

/**
 * extends {@link JPanel}<br>
 * implements {@link MouseListener}<br>
 * <br>
 * Create an accordion panel open and close with a mouse click.
 */
public class ExpansionContractionPanel extends JPanel implements
                                                     MouseListener {
    /** serialVersionUID. */
    private static final long serialVersionUID = -7418609214667862469L;

    /** Title String. */
    private String mainTitle;

    /** Title Label. */
    private JLabel titleLbl;

    /** Main Panel. */
    private JPanel mainPnl;

    /**
     * Create the accordion panel.
     *
     * @param title the panel title.
     */
    public ExpansionContractionPanel(final String title) {
        super(new BorderLayout());

        mainTitle = title;
        titleLbl = new JLabel();
        mainPnl = new JPanel(new BorderLayout());
        titleLbl.setOpaque(true);
        titleLbl.setBackground(Color.GRAY);
        titleLbl.addMouseListener(this);
        titleLbl.setForeground(Color.WHITE);
        add(titleLbl, BorderLayout.NORTH);
    }

    /**
     * Set the component.
     *
     * @param main main component
     * @param visible true to make the component visible; false to make it
     *        invisible
     */
    public final void setComponent(final JComponent main, final boolean visible) {
        mainPnl.removeAll();
        mainPnl.add(main, BorderLayout.CENTER);
        mainPnl.setVisible(visible);
        mainPnl.setOpaque(true);
        mainPnl.setBackground(Color.WHITE);
        Border outBorder = BorderFactory.createMatteBorder(0, 2, 2, 2,
                Color.GRAY);
        Border inBorder = BorderFactory.createEmptyBorder(
                GuiCommon.BORDER_SIZE, GuiCommon.BORDER_SIZE,
                GuiCommon.BORDER_SIZE, GuiCommon.BORDER_SIZE);
        Border border = BorderFactory.createCompoundBorder(outBorder, inBorder);
        mainPnl.setBorder(border);
        if (mainPnl.isVisible()) {
            titleLbl.setText(" \u25b3 " + mainTitle);
        } else {
            titleLbl.setText(" \u25bc " + mainTitle);
        }
        add(mainPnl, BorderLayout.CENTER);
    }

    /**
     * Set the title of this component.
     *
     * @param title label title
     */
    public final void setTitle(final String title) {
        mainTitle = title;
    }

    /**
     * Perform the initialization of this component.
     */
    protected final void initPanel() {
        mainPnl.setVisible(!mainPnl.isVisible());
        if (mainPnl.isVisible()) {
            titleLbl.setText(" \u25b3 " + mainTitle);
        } else {
            titleLbl.setText(" \u25bc " + mainTitle);
        }
        revalidate();
        mainPnl.scrollRectToVisible(mainPnl.getBounds());
    }

    @Override
    public final Dimension getPreferredSize() {
        Dimension size = titleLbl.getPreferredSize();
        if (mainPnl.isVisible()) {
            size.height += mainPnl.getPreferredSize().height;
        }
        return size;
    }

    @Override
    public final Dimension getMaximumSize() {
        Dimension size = getPreferredSize();
        size.width = Short.MAX_VALUE;
        return size;
    }

    @Override
    public void mouseClicked(final MouseEvent arg0) {
    }

    @Override
    public void mouseEntered(final MouseEvent arg0) {
    }

    @Override
    public void mouseExited(final MouseEvent arg0) {
    }

    @Override
    public final void mousePressed(final MouseEvent arg0) {
        initPanel();
    }

    @Override
    public void mouseReleased(final MouseEvent arg0) {
    }
}
